@@ -5,9 +5,12 @@ use 5.005;
 use strict;
 use vars qw($VERSION $BASE_EXC_CLASS %CLASSES);
 
+use Scalar::Util qw(blessed);
+
+
 BEGIN { $BASE_EXC_CLASS ||= 'Exception::Class::Base'; }
 
-$VERSION = '1.20';
+$VERSION = '1.24';
 
 sub import
 {
@@ -20,8 +23,8 @@ sub import
     my %needs_parent;
     while (my $subclass = shift)
     {
-	my $def = ref $_[0] ? shift : {};
-	$def->{isa} = $def->{isa} ? ( ref $def->{isa} ? $def->{isa} : [$def->{isa}] ) : [];
+        my $def = ref $_[0] ? shift : {};
+        $def->{isa} = $def->{isa} ? ( ref $def->{isa} ? $def->{isa} : [$def->{isa}] ) : [];
 
         $c{$subclass} = $def;
     }
@@ -33,32 +36,32 @@ sub import
     {
         my $def = $c{$subclass};
 
-	# We already made this one.
-	next if $CLASSES{$subclass};
+        # We already made this one.
+        next if $CLASSES{$subclass};
 
-	{
-	    no strict 'refs';
-	    foreach my $parent (@{ $def->{isa} })
-	    {
-		unless ( keys %{"$parent\::"} )
-		{
-		    $needs_parent{$subclass} = { parents => $def->{isa},
-						 def => $def };
-		    next MAKE_CLASSES;
-		}
-	    }
-	}
+        {
+            no strict 'refs';
+            foreach my $parent (@{ $def->{isa} })
+            {
+                unless ( keys %{"$parent\::"} )
+                {
+                    $needs_parent{$subclass} = { parents => $def->{isa},
+                                                 def => $def };
+                    next MAKE_CLASSES;
+                }
+            }
+        }
 
-	$class->_make_subclass( subclass => $subclass,
-				def => $def || {},
+        $class->_make_subclass( subclass => $subclass,
+                                def => $def || {},
                               );
     }
 
     foreach my $subclass (keys %needs_parent)
     {
-	# This will be used to spot circular references.
-	my %seen;
-	$class->_make_parents( \%needs_parent, $subclass, \%seen );
+        # This will be used to spot circular references.
+        my %seen;
+        $class->_make_parents( \%needs_parent, $subclass, \%seen );
     }
 }
 
@@ -79,25 +82,25 @@ sub _make_parents
     # mentioned is in the 'isa' param for some other class, which is
     # not a good enough reason to make a new class.
     die "Class $subclass appears to be a typo as it is only specified in the 'isa' param for $child\n"
-	unless exists $needs->{$subclass} || $CLASSES{$subclass} || keys %{"$subclass\::"};
+        unless exists $needs->{$subclass} || $CLASSES{$subclass} || keys %{"$subclass\::"};
 
     foreach my $c ( @{ $needs->{$subclass}{parents} } )
     {
-	# It's been made
-	next if $CLASSES{$c} || keys %{"$c\::"};
+        # It's been made
+        next if $CLASSES{$c} || keys %{"$c\::"};
 
-	die "There appears to be some circularity involving $subclass\n"
-	    if $seen->{$subclass};
+        die "There appears to be some circularity involving $subclass\n"
+            if $seen->{$subclass};
 
-	$seen->{$subclass} = 1;
+        $seen->{$subclass} = 1;
 
-	$class->_make_parents( $needs, $c, $seen, $subclass );
+        $class->_make_parents( $needs, $c, $seen, $subclass );
     }
 
     return if $CLASSES{$subclass} || keys %{"$subclass\::"};
 
     $class->_make_subclass( subclass => $subclass,
-			    def => $needs->{$subclass}{def} );
+                            def => $needs->{$subclass}{def} );
 }
 
 sub _make_subclass
@@ -111,7 +114,7 @@ sub _make_subclass
     my $isa;
     if ($def->{isa})
     {
-	$isa = ref $def->{isa} ? join ' ', @{ $def->{isa} } : $def->{isa};
+        $isa = ref $def->{isa} ? join ' ', @{ $def->{isa} } : $def->{isa};
     }
     $isa ||= $BASE_EXC_CLASS;
 
@@ -131,8 +134,8 @@ EOPERL
 
     if ($def->{description})
     {
-	(my $desc = $def->{description}) =~ s/([\\\'])/\\$1/g;
-	$code .= <<"EOPERL";
+        (my $desc = $def->{description}) =~ s/([\\\'])/\\$1/g;
+        $code .= <<"EOPERL";
 sub description
 {
     return '$desc';
@@ -143,16 +146,16 @@ EOPERL
     my @fields;
     if ( my $fields = $def->{fields} )
     {
-	@fields = UNIVERSAL::isa($fields, 'ARRAY') ? @$fields : $fields;
+        @fields = UNIVERSAL::isa($fields, 'ARRAY') ? @$fields : $fields;
 
-	$code .=
+        $code .=
             "sub Fields { return (\$_[0]->SUPER::Fields, " .
             join(", ", map { "'$_'" } @fields) . ") }\n\n";
 
         foreach my $field (@fields)
-	{
-	    $code .= sprintf("sub %s { \$_[0]->{%s} }\n", $field, $field);
-	}
+        {
+            $code .= sprintf("sub %s { \$_[0]->{%s} }\n", $field, $field);
+        }
     }
 
     if ( my $alias = $def->{alias} )
@@ -171,12 +174,22 @@ EOPERL
     $CLASSES{$subclass} = 1;
 }
 
+sub caught
+{
+    my $e = $@;
+
+    return $e unless $_[1];
+
+    return unless blessed($e) && $e->isa( $_[1] );
+    return $e;
+}
+
 sub Classes { sort keys %Exception::Class::CLASSES }
 
 package Exception::Class::Base;
 
 use Class::Data::Inheritable;
-use Devel::StackTrace 1.07;
+use Devel::StackTrace 1.17;
 
 use base qw(Class::Data::Inheritable);
 
@@ -191,6 +204,8 @@ BEGIN
     __PACKAGE__->mk_classdata('RespectOverload');
     __PACKAGE__->RespectOverload(0);
 
+    __PACKAGE__->mk_classdata('MaxArgLength');
+    __PACKAGE__->MaxArgLength(0);
 
     sub Fields { () }
 }
@@ -213,7 +228,7 @@ BEGIN
     no strict 'refs';
     foreach my $f (@fields)
     {
-	*{$f} = sub { my $s = shift; return $s->{$f}; };
+        *{$f} = sub { my $s = shift; return $s->{$f}; };
     }
     *{'error'} = \&message;
 }
@@ -287,6 +302,7 @@ sub _initialize
                                 ignore_package   => \@ignore_package,
                                 no_refs          => $self->NoRefs,
                                 respect_overload => $self->RespectOverload,
+                                max_arg_length   => $self->MaxArgLength,
                               );
 
     if ( my $frame = $self->trace->frame(0) )
@@ -367,6 +383,12 @@ sub isa
 }
 EOF
 
+sub caught
+{
+    return Exception::Class->caught(shift);
+}
+
+
 1;
 
 __END__
@@ -391,26 +413,29 @@ Exception::Class - A module that allows you to declare real exception classes in
         { isa => 'YetAnotherException',
           fields => [ 'grandiosity', 'quixotic' ],
           alias => 'throw_fields',
+        },
       );
 
   # try
   eval { MyException->throw( error => 'I feel funny.' ) };
 
+  my $e;
   # catch
-  if ( UNIVERSAL::isa( $@, 'MyException' ) )
+  if ( $e = Exception::Class->caught('MyException') )
   {
-     warn $@->error, "\n, $@->trace->as_string, "\n";
-     warn join ' ',  $@->euid, $@->egid, $@->uid, $@->gid, $@->pid, $@->time;
+     warn $e->error, "\n", $e->trace->as_string, "\n";
+     warn join ' ',  $e->euid, $e->egid, $e->uid, $e->gid, $e->pid, $e->time;
 
      exit;
   }
-  elsif ( UNIVERSAL::isa( $@, 'ExceptionWithFields' ) )
+  elsif ( $e = Exception::Class->caught('ExceptionWithFields') )
   {
-     $@->quixotic ? do_something_wacky() : do_something_sane();
+     $e->quixotic ? do_something_wacky() : do_something_sane();
   }
   else
   {
-     ref $@ ? $@->rethrow : die $@;
+     $e = Exception::Class->caught();
+     ref $e ? $e->rethrow : die $e;
   }
 
   # use an alias - without parens subroutine name is checked at
@@ -520,6 +545,51 @@ hierarchies and will die if it finds one.  It also detects missing
 links in a chain, for example if you declare Bar to be a subclass of
 Foo and never declare Foo.
 
+=head1 Catching Exceptions
+
+C<Exception::Class> provides some syntactic sugar for catching
+exceptions in a safe manner:
+
+ eval { ... }
+
+ if ( my $e = Exception::Class->caught('My::Error') )
+ {
+     cleanup();
+     do_something_with_exception($e);
+ }
+
+The C<caught()> method takes a class name and returns an exception
+object if the last thrown exception is of the given class, or a
+subclass of that class.  If it is not given any arguments, it simply
+returns C<$@>.
+
+You should B<always> make a copy of the exception object, rather than
+using C<$@> directly.  This is necessary because if your C<cleanup()>
+function uses C<eval>, or calls something which uses it, then C<$@> is
+overwritten.  Copying the exception preserves it for the call to
+C<do_something_with_exception()>.
+
+Exception objects also provide a caught method so you can write:
+
+ if ( my $e = My::Error->caught() )
+ {
+     cleanup();
+     do_something_with_exception($e);
+ }
+
+=head2 Uncatchable Exceptions
+
+Internally, the C<caught()> method will call C<isa()> on the exception
+object.  You could make an exception "uncatchable" by overriding
+C<isa()> in that class like this:
+
+ package Exception::Uncatchable;
+
+ sub isa { shift->rethrow }
+
+Of course, this only works if you always call 
+C<< Exception::Class->caught() >> after an C<eval>.
+
 =head1 Exception::Class::Base CLASS METHODS
 
 =over 4
@@ -568,6 +638,20 @@ This method defaults to false.  As with C<Trace()>, it is inherited by
 subclasses but setting it in a subclass makes it independent
 thereafter.
 
+=item * MaxArgLength($boolean)
+
+When a C<Devel::StackTrace> object stringifies, by default it displays
+the full argument for each function. This parameter can be used to
+limit the maximum length of each argument.
+
+Since C<Exception::Class::Base> uses C<Devel::StackTrace> internally,
+this method provides a way to tell C<Devel::StackTrace> to limit the
+length of arguments.
+
+This method defaults to 0. As with C<Trace()>, it is inherited by
+subclasses but setting it in a subclass makes it independent
+thereafter.
+
 =item * Fields
 
 This method returns the extra fields defined for the given class, as
@@ -703,7 +787,7 @@ overridden by a subclass.  See below for details.
 The C<Exception::Class::Base> object is overloaded so that
 stringification produces a normal error message.  It just calls the
 as_string method described above.  This means that you can just
-C<print $@> after an eval and not worry about whether or not its an
+C<print $@> after an C<eval> and not worry about whether or not its an
 actual object.  It also means an application or module could do this:
 
  $SIG{__DIE__} = sub { Exception::Class::Base->throw( error => join '', @_ ); };
@@ -804,21 +888,23 @@ somewhere:
 
 It's a hack but apparently it works.
 
+=head1 SUPPORT
+
+Please submit bugs to the CPAN RT system at
+http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Exception%3A%3AClass or
+via email at bug-exception-class@rt.cpan.org.
+
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
 
-=head1 SEE ALSO
+=head1 COPYRIGHT
 
-Devel::StackTrace - used by this module to create stack traces
+Copyright (c) 2000-2006 David Rolsky.  All rights reserved.  This
+program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
-Error.pm - implements try/catch in Perl.  Also provides an exception
-base class.
-
-Test::Exception - a module that helps you test exception based code.
-
-Numerous other modules/frameworks seem to have their own exception
-classes (SPOPS and Template Toolkit, to name two) but none of these
-seem to be designed for use outside of these packages.
+The full text of the license can be found in the LICENSE file included
+with this module.
 
 =cut
